@@ -7,7 +7,7 @@ import numpy
 # Locals
 from .phi import phi
 
-def InitialRBFMatrices(PairwiseDistance,samples,phifunction='linear',
+def InitialRBFMatrices(PairwiseDistance,samples,phifunction='cubic',
                        polynomial='linear',**kwargs):
     PHI = numpy.zeros((samples.shape[0], samples.shape[0]))
     if phifunction == 'linear':
@@ -35,14 +35,14 @@ def InitialRBFMatrices(PairwiseDistance,samples,phifunction='linear',
         raise myException('Error: Invalid polynomial tail.')
     return numpy.asmatrix(PHI), numpy.asmatrix(P), pdim
 
-def UpdateRBFMatrices(samples, PHI, P, PairwiseDistance, r, phifunction='linear', polynomial='linear', **kwargs):
+def UpdateRBFMatrices(samples, PHI, P, PairwiseDistance, r, phifunction='cubic', polynomial='linear', **kwargs):
     phi0 = phi(0, phifunction)
     new_phi = phi(r, phifunction).squeeze()
     PHI_new = numpy.zeros((len(samples), len(samples)))
     PHI_new[:PHI.shape[0],:PHI.shape[1]] = numpy.array(PHI)
+    PHI_new[-1,0:new_phi.shape[1]] = new_phi
+    PHI_new[0:new_phi.shape[1],-1] = new_phi
     PHI_new[-1,-1] = phi0
-    PHI_new[-1,0:-1] = new_phi
-    PHI_new[0:-1,-1] = new_phi
     if polynomial == 'None':
         P = numpy.array([])
     elif polynomial == 'constant':
