@@ -16,7 +16,7 @@ from .datasets import get_loader
 from .dnnmodels import training, run_uq
 from .hyperparams import get_hyperprms
 from .utils.extract import gather_trials, check_evaluation_outer_loss
-from .obj_fct import *
+from .obj_fct import calculate_outer_loss
 
 def train_evaluation(x_sc, samples, trainer, model, dist, output='out', data={}, uq={}, **kwargs):
     """
@@ -56,47 +56,42 @@ def train_evaluation(x_sc, samples, trainer, model, dist, output='out', data={},
     trials = numpy.arange(model['trial'])
     if dist['split']=='trial':
         trials = trials[dist['rank']::dist['size']]
+    loaders = get_loader(data, **hyperprms, **model, **dist) if trainer=='internal' else None
     # Loop through all selected trials
-    all_results = []
     for i in trials:
         out_path = '%s/%02i' % (output,i+1)
         if trainer=='internal':
-            loaders = get_loader(data, **hyperprms, **model, **dist)
-            results = training(itrial=i,data=loaders,hyperprms=hyperprms,output=out_path,**model,**dist)
+            loss = training(itrial=i,data=loaders,hyperprms=hyperprms,output=out_path,**model,**dist)
         else:
-            results = external_training(i, x_sc, trainer, out_path, model, **dist)
-        all_results.append(results)
-    # Prepare single results dictionary
-    results = all_results[0]
-    results['output'] = output
-    for key in ['loss','loss_tf','models']:
-        results[key] = [model_results[key] for model_results in all_results]
+            loss = external_training(i, x_sc, trainer, out_path, model, **dist)
     # Check if all losses over all trials are found
     res = None
     if dist['rank']==0:
-        if dist['ntasks']>1:
-            while len(gather_trials(results,samples,out_path,**model,**dist)['loss']) != model['trial']:
-                continue
-            results = gather_trials(results,samples,out_path,**model,**dist)
-        # Remove trials that gave nan values
-        idxs = numpy.argwhere(numpy.isnan(results['loss']))
-        for key in ['loss','loss_tf','models']:
-            results[key] = numpy.delete(results[key],idxs)
-        # Execute UQ or not
+        while len(gather_trials(samples,output,**model,**dist)) != model['trial']:
+            continue
+        losses = gather_trials(samples,output,**model,**dist)
         if 'uq_on' in uq.keys() and uq['uq_on']==True:
-            res = run_uq(**results,**model,**dist,**uq)
+            valid_trials = numpy.argwhere(~numpy.isnan(losses)).squeeze()
+            res = run_uq(trials=valid_trials,hyperprms=hyperprms,data=loaders,output=output,loss=losses,**model,**dist,**uq)
         else:
+            if numpy.isnan(losses).all():
+                outer_loss = float('nan')
+                inner_loss = float('nan')
+                std        = float('nan')
+            else:
+                outer_loss = numpy.nanmean(calculate_outer_loss(losses,**model))
+                inner_loss = numpy.nanmean(losses)
+                std        = numpy.nanstd(losses)
             logging.info('-'*40)
-            logging.info('OUTER OBJECTIVE FUNCTION {:>8} TRIALS'.format(len(results['loss'])))
+            logging.info('OUTER OBJECTIVE FUNCTION {:>8} TRIALS'.format(len(losses)))
             logging.info('-'*40)
-            logging.info('\tOuter Loss : {:>11.5f}'.format(numpy.mean(results['loss_tf'])))
-            logging.info('\tInner Loss : {:>11.5f}'.format(numpy.mean(results['loss'])))
-            logging.info('\tSTD        : {:>11.5f}'.format(numpy.std(results['loss'])))
-            res = numpy.mean(results['loss_tf'])
+            logging.info('\tOuter Loss : {:>11.5f}'.format(outer_loss))
+            logging.info('\tInner Loss : {:>11.5f}'.format(inner_loss))
+            logging.info('\tSTD        : {:>11.5f}'.format(std))
     else:
         logging.info('-'*40)
         logging.info('Waiting for other processes to finish...')
-        while check_evaluation_outer_loss(samples,out_path,**dist)==False:
+        while check_evaluation_outer_loss(samples,output,**dist)==False:
             continue
     logging.info('-'*40)
     logging.info("Execution Time : %.3f s" % (time.time() - start_time)) 
@@ -105,7 +100,7 @@ def train_evaluation(x_sc, samples, trainer, model, dist, output='out', data={},
 def external_training(itrial, x_sc, trainer, output_dir, model, log_dir, **kwargs):
     start_time = time.time()
     logging.info('-'*40)
-    logging.info('{} {:>3} / {:<3} {:>24}'.format('TRIAL',itrial+1,model['trial'],'TESTING'))
+    logging.info('{} {:>3} / {:<3} {:>24}'.format('TRIAL',itrial+1,model['trial'],'TRAINING'))
     logging.info('-'*40)
     model['output_dir'] = os.path.join(log_dir,'output',output_dir)
     if 'yaml_input' in model.keys():
@@ -118,9 +113,11 @@ def external_training(itrial, x_sc, trainer, output_dir, model, log_dir, **kwarg
         modlist = trainer.split('.')
         module = importlib.import_module('.' + modlist[-2], '.'.join(modlist[:-2]))
         loss = eval('module.'+modlist[-1])(**params)
-    logging.info('\t\tLoss : {:>11.5f}'.format(loss))
-    logging.info('\t\tTime : {:>11.5f} s'.format(time.time()-start_time))
-    return {'loss':loss,'loss_tf':loss,'models':None}
+    logging.info('-'*40)
+    logging.info('{} {:>3} / {:<3} {:>24}'.format('TRIAL',itrial+1,model['trial'],'TESTING'))
+    logging.info('-'*40)
+    logging.info('\t Test Loss : {:>11.5f}'.format(loss))
+    logging.info('\tTrain Time : {:>11.5f} s'.format(time.time()-start_time))
 
 def yaml_creation(x_sc,output_dir,src,config=None,**kwargs):
     # Extract input configuration file
