@@ -3,6 +3,7 @@ import os
 import ast
 import glob
 import math
+import logging
 
 # External
 import yaml
@@ -11,7 +12,7 @@ import pandas as pd
 import pickle5 as pickle
 from scipy import interpolate
 
-def get_convergence(search_dir,with_surrogate=False,sigma=False,niters=None,nruns=None,debug=False,**kwargs):
+def get_convergence(search_dir,with_surrogate=False,werr=True,niters=None,nruns=None,debug=False,**kwargs):
     dir_list = sorted(glob.glob(search_dir+'/*'))
     if any('.log' in dir_name for dir_name in dir_list):
         dir_list = [search_dir]
@@ -50,7 +51,7 @@ def get_convergence(search_dir,with_surrogate=False,sigma=False,niters=None,nrun
         all_results[1][i] = all_results[1][i][:niters]
     all_results = numpy.array(all_results)
     mean_loss = numpy.mean(all_results[0],axis=0)
-    if sigma:
+    if werr:
         mean_sdev = numpy.sqrt(numpy.sum(all_results[1]**2,axis=0))/all_results[0].shape[0]
     else:
         mean_sdev = numpy.std(all_results[0],axis=0)
@@ -101,7 +102,12 @@ def check_surrogate_sample(log_dir,step,iloop):
     samples = []
     log_file = './%s/surrogate_%03i_01.log' % (log_dir,step+1)
     if os.path.exists(log_file):
-        log_content = numpy.loadtxt(log_file,dtype=str,delimiter='\n').tolist()
+        while True:
+            try:
+                log_content = numpy.loadtxt(log_file,dtype=str,delimiter='\n').tolist()
+                break
+            except:
+                pass
         for i,line in enumerate(log_content):
             if 'SURROGATE ITERATION' in line:
                 isurrogate = int(line.split()[-3])
@@ -115,7 +121,12 @@ def check_evaluation_outer_loss(target,output,log_dir,step,**kwargs):
     outer_loss = False
     log_file = './%s/%s_%03i_01.log' % (log_dir,output.split('_')[0],step+1)
     if os.path.exists(log_file):
-        log_content = numpy.loadtxt(log_file,dtype=str,delimiter='\n').tolist()
+        while True:
+            try:
+                log_content = numpy.loadtxt(log_file,dtype=str,delimiter='\n').tolist()
+                break
+            except:
+                pass
         for i,line in enumerate(log_content):
             if 'Samples:' in line:
                 samples = line.split('Samples:')[-1].strip()[1:-1]
@@ -141,7 +152,12 @@ def gather_trials(target,output,log_dir,ntasks,step,trial,split,**kwargs):
     n=0
     for rank in range(ntasks):
         log_file = './%s/%s_%03i_%02i.log' % (log_dir,output.split('_')[0],step+1,rank+1)
-        log_content = numpy.loadtxt(log_file,dtype=str,delimiter='\n').tolist()
+        while True:
+            try:
+                log_content = numpy.loadtxt(log_file,dtype=str,delimiter='\n').tolist()
+                break
+            except:
+                pass
         for i,line in enumerate(log_content):
             if 'Samples:' in line:
                 samples = line.split('Samples:')[-1].strip()[1:-1]
