@@ -17,6 +17,7 @@ from matplotlib import ticker, cm
 from matplotlib.colors import LogNorm
 from matplotlib.ticker import FormatStrFormatter
 from mpl_toolkits.mplot3d import Axes3D
+from plotly.subplots import make_subplots
 
 # Locals
 from .extract import *
@@ -24,7 +25,7 @@ from ..dnnmodels import training
 from ..datasets import get_data
 from ..hyperparams import set_hyperparams
 
-def plot_convergence(loss,stdev,isurr=None,ylim=[],xlim=[]):
+def plot_convergence(loss,stdev,isurr=None,ylim=[],xlim=[],save=None):
     plt.style.use('seaborn')
     plt.figure(figsize=(6,4),dpi=100)
     plt.plot(loss,color='black',lw=0.5,zorder=1,
@@ -40,27 +41,55 @@ def plot_convergence(loss,stdev,isurr=None,ylim=[],xlim=[]):
         plt.xlim(xlim)
     else:
         plt.xlim(0,len(loss))
-    plt.gca().yaxis.set_major_formatter(FormatStrFormatter('%.2f'))
     plt.xlabel('Index of function evaluations')
     plt.ylabel('Loss')
     plt.legend(loc='upper right')
     plt.tight_layout()
-    plt.show()
+    if save==None:
+        plt.show()
+    else:
+        plt.savefig(save+'.pdf')
+        plt.close()
 
-def hpo_solutions(results):
+def get_custom_data(results,config=None,sensitivity=False):
+    hp_names = []
+    customdata = results[['params','loss','stdev']].to_numpy().T
+    text_coord = \
+        '<b>Model Results:</b><br>' + \
+        'Mean Loss of %{customdata[1]:.5f}<br>' + \
+        'Uncertainty of %{customdata[2]:.5f}<br>' + \
+        '%{customdata[0]:,} trainable parameters<br><br>' + \
+        '<b>Hyperparameter Set:</b><br>'
+    if config!=None:
+        with open(config) as f:
+            config = yaml.load(f, Loader=yaml.FullLoader)
+    for hp_name in results.columns:
+        hp_fmt, hp_vals = 'i', results[hp_name].to_numpy()
+        if hp_name.startswith('hp'):
+            hp_idx = hp_name[2:]
+            if type(config)==dict:
+                scale = config['prms']['mult'][int(hp_idx)-1]
+                hp_name = config['prms']['names'][int(hp_idx)-1]
+                hp_vals = hp_vals*float(scale)
+                if '.' in str(scale):
+                    hp_fmt = '.%if' % len(str(scale).split('.')[-1])
+            text_coord += (hp_name+' = %{customdata['+str(int(hp_idx)+2)+']:'+hp_fmt+'}<br>')
+            customdata = numpy.vstack((customdata,hp_vals))
+            hp_names.append(hp_name)
+    text_coord += '<extra></extra>'
+    return text_coord, customdata.T, hp_names
+
+def hpo_solutions(results,ms=20,edgewidth=2,config=None,log=True):
+    text_coord, custom_data, _ = get_custom_data(results,config)
     fig = go.Figure(
         data=go.Scatter(
             x=results.loss,
             y=results.stdev,
             mode='markers',
-            customdata=results.params.to_numpy(),
-            hovertemplate= \
-            'Mean Loss of %{x:.5f}<br>' + \
-            'Uncertainty of %{y:.5f}<br>' + \
-            '%{customdata:,} trainable parameters<br>' + \
-            '<extra></extra>',
+            customdata=custom_data,
+            hovertemplate=text_coord,
             marker=dict(
-                size=15,
+                size=ms,
                 opacity=0.5,
                 color=results.params,
                 colorbar=dict(
@@ -68,11 +97,16 @@ def hpo_solutions(results):
                     title='Number of trainable parameters',
                     titleside='right'
                 ),
+                line=dict(
+                    color='black',
+                    width=edgewidth,
+                ),
             ),
         )
     )
-    fig.update_xaxes(type='log')
-    fig.update_yaxes(type='log')
+    if log:
+        fig.update_xaxes(type='log')
+        fig.update_yaxes(type='log')
     fig.update_layout(
         autosize=True,
         xaxis_title='Loss',
@@ -83,6 +117,78 @@ def hpo_solutions(results):
     fig.write_html('solutions.html')
     fig.show()
 
+def hpo_sensitivity(results,ms=20,edgewidth=2,ncols=3,config=None,log=False):
+    """
+    Parameters
+    ----------
+    results : :class:`pandas.core.frame.DataFrame`
+      Extracted solutions from HYPPO log product files.
+    ms : :class:`int`
+      Marker size
+    edgewidth : :class:`float`
+      Marker edge line width
+    ncols : :class:`int`
+      Number of columns for multiplot figure
+    config : :class:`str`
+      Path to configuration file. If None, no scaling will be done
+      and the hyperparameter name will be hp1, hp2,...
+    log : :class:`bool`
+      Display loss axes in logarithmic scale
+
+    Examples
+    --------
+    >>> import hyppo
+    >>> solutions = hyppo.extract('logs/*.log')
+    >>> hyppo.hpo_sensitivity(solutions,config='config.yaml')
+    
+    .. raw:: html
+       
+       <script src="https://cdn.plot.ly/plotly-latest.min.js"></script>
+       <embed type="text/html" src="../_static/plotly/sensitivity.html" width="100%" height="600">
+    """
+    text_coord, custom_data, hp_names = get_custom_data(results,config,sensitivity=True)
+    n_hps = custom_data.shape[1]-3
+    nrows = int(numpy.ceil(n_hps/ncols))
+    fig = make_subplots(rows=nrows, cols=ncols, start_cell="top-left",
+                        horizontal_spacing=0.03, vertical_spacing=0.08)
+    for n,(irow,jcol) in enumerate([(i+1,j+1) for i in range(nrows) for j in range(ncols)]):
+        if n+1>n_hps:
+            break
+        fig.add_trace(
+            go.Scatter(
+                x=custom_data[:,3+n],
+                y=results.loss,
+                mode='markers',
+                customdata=custom_data,
+                hovertemplate=text_coord,
+                marker=dict(
+                    size=ms,
+                    opacity=0.5,
+                    color=results.stdev,
+                    colorbar=dict(
+                        thickness=20,
+                        title='1-'+u'\u03C3'+' Deviation',
+                        titleside='right'
+                    ),
+                    line=dict(
+                        color='black',
+                        width=edgewidth,
+                    ),
+                ),
+            ),row=irow, col=jcol)
+        fig['layout']['xaxis%i' % (n+1)]['title'] = hp_names[n]
+        if n%3==0:
+            fig['layout']['yaxis%i' % (n+1)]['title'] = 'Loss'
+    if log:
+        fig.update_yaxes(type='log')
+    fig.update_layout(
+        autosize=True,
+        margin=dict(l=0, r=0, b=0, t=0),
+        showlegend=False,
+    )
+    fig.write_html('sensitivity.html')
+    fig.show()
+
 def model_visualization_1d(X,Y,path,evaluation,surrogate,imax=10,sigma=False,ylim=[-0.5,2.5]):
     scaler = round(1/(X[1]-X[0]))
     os.makedirs('%s/plots' % (path),exist_ok=True)
